@@ -27,7 +27,6 @@
 
 #include <gjs/gjs-module.h>
 #include <gjs/compat.h>
-#include <gjs/runtime.h>
 #include "boxed.h"
 #include "enumeration.h"
 #include "repo.h"
@@ -85,7 +84,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(error)
                         "GError constructor, obj %p priv %p",
                         object, priv);
 
-    proto = JS_GetPrototype(object);
+    JS_GetPrototype(context, object, &proto);
     gjs_debug_lifecycle(GJS_DEBUG_GERROR, "GError instance __proto__ is %p", proto);
 
     /* If we're the prototype, then post-construct we'll fill in priv->info.
@@ -103,10 +102,8 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(error)
     g_base_info_ref( (GIBaseInfo*) priv->info);
     priv->domain = proto_priv->domain;
 
-    message_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                                GJS_STRING_MESSAGE);
-    code_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                             GJS_STRING_CODE);
+    message_name = gjs_context_get_const_string(context, GJS_STRING_MESSAGE);
+    code_name = gjs_context_get_const_string(context, GJS_STRING_CODE);
     if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(argv[0]),
                                      "GError constructor", message_name, &v_message))
         return JS_FALSE;
@@ -274,8 +271,7 @@ error_constructor_value_of(JSContext *context, unsigned argc, jsval *vp)
         return JS_FALSE;
     }
 
-    prototype_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                                  GJS_STRING_PROTOTYPE);
+    prototype_name = gjs_context_get_const_string(context, GJS_STRING_PROTOTYPE);
     if (!gjs_object_require_property(context,
                                      JSVAL_TO_OBJECT(v_self),
                                      "constructor",
@@ -310,7 +306,7 @@ struct JSClass gjs_error_class = {
     JSCLASS_HAS_PRIVATE |
     JSCLASS_NEW_RESOLVE,
     JS_PropertyStub,
-    JS_PropertyStub,
+    JS_DeletePropertyStub,
     JS_PropertyStub,
     JS_StrictPropertyStub,
     JS_EnumerateStub,
@@ -452,12 +448,9 @@ define_error_properties(JSContext *context,
                                      &lineNumber))
         return;
 
-    stack_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                              GJS_STRING_STACK);
-    filename_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                                 GJS_STRING_FILENAME);
-    linenumber_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                                   GJS_STRING_LINE_NUMBER);
+    stack_name = gjs_context_get_const_string(context, GJS_STRING_STACK);
+    filename_name = gjs_context_get_const_string(context, GJS_STRING_FILENAME);
+    linenumber_name = gjs_context_get_const_string(context, GJS_STRING_LINE_NUMBER);
 
     JS_DefinePropertyById(context, obj, stack_name, stack,
                           NULL, NULL, JSPROP_ENUMERATE);
