@@ -42,7 +42,6 @@
 #include <gjs/gjs-module.h>
 #include <gjs/compat.h>
 #include <gjs/type-module.h>
-#include <gjs/runtime.h>
 
 #include <util/log.h>
 #include <util/hash-x32.h>
@@ -76,7 +75,6 @@ typedef enum
 
 typedef struct
 {
-    JSContext       *context;
     GObject         *gobj;
     ToggleDirection  direction;
     guint            needs_unref : 1;
@@ -240,7 +238,9 @@ static inline ObjectInstance *
 proto_priv_from_js(JSContext *context,
                    JSObject  *obj)
 {
-    return priv_from_js(context, JS_GetPrototype(obj));
+    JSObject *proto;
+    JS_GetPrototype(context, obj, &proto);
+    return priv_from_js(context, proto);
 }
 
 /* a hook on getting a property; set value_p to override property's value.
@@ -394,11 +394,13 @@ is_vfunc_unchanged(GIVFuncInfo *info,
 }
 
 static GIVFuncInfo *
-find_vfunc_on_parent(GIObjectInfo *info,
-                     gchar        *name)
+find_vfunc_on_parents(GIObjectInfo *info,
+                      gchar        *name,
+                      gboolean     *out_defined_by_parent)
 {
     GIVFuncInfo *vfunc = NULL;
     GIObjectInfo *parent;
+    gboolean defined_by_parent = FALSE;
 
     /* ref the first info so that we don't destroy
      * it when unrefing parents later */
@@ -415,11 +417,16 @@ find_vfunc_on_parent(GIObjectInfo *info,
         g_base_info_unref(tmp);
         if (parent)
             vfunc = g_object_info_find_vfunc(parent, name);
+
+        defined_by_parent = TRUE;
     }
 
     if (parent)
         g_base_info_unref(parent);
 
+    if (out_defined_by_parent)
+        *out_defined_by_parent = defined_by_parent;
+
     return vfunc;
 }
 
@@ -559,12 +566,13 @@ object_instance_new_resolve(JSContext *context,
 
         gchar *name_without_vfunc_ = &name[6];
         GIVFuncInfo *vfunc;
+        gboolean defined_by_parent;
 
-        vfunc = find_vfunc_on_parent(priv->info, name_without_vfunc_);
+        vfunc = find_vfunc_on_parents(priv->info, name_without_vfunc_, &defined_by_parent);
         if (vfunc != NULL) {
             /* In the event that the vfunc is unchanged, let regular
              * prototypal inheritance take over. */
-            if (is_vfunc_unchanged(vfunc, priv->gtype)) {
+            if (defined_by_parent && is_vfunc_unchanged(vfunc, priv->gtype)) {
                 g_base_info_unref((GIBaseInfo *)vfunc);
                 ret = JS_TRUE;
                 goto out;
@@ -829,15 +837,14 @@ cancel_toggle_idle(GObject         *gobj,
 }
 
 static void
-handle_toggle_down(JSContext *context,
-                   GObject   *gobj)
+handle_toggle_down(GObject *gobj)
 {
     ObjectInstance *priv;
     JSObject *obj;
 
     obj = peek_js_obj(gobj);
 
-    priv = priv_from_js(context, obj);
+    priv = (ObjectInstance *) JS_GetPrivate(obj);
 
     gjs_debug_lifecycle(GJS_DEBUG_GOBJECT,
                         "Toggle notify gobj %p obj %p is_last_ref TRUE keep-alive %p",
@@ -848,7 +855,7 @@ handle_toggle_down(JSContext *context,
      */
     if (priv->keep_alive != NULL) {
         gjs_debug_lifecycle(GJS_DEBUG_GOBJECT, "Removing object from keep alive");
-        gjs_keep_alive_remove_child(context, priv->keep_alive,
+        gjs_keep_alive_remove_child(priv->keep_alive,
                                     gobj_no_longer_kept_alive_func,
                                     obj,
                                     priv);
@@ -857,8 +864,7 @@ handle_toggle_down(JSContext *context,
 }
 
 static void
-handle_toggle_up(JSContext *context,
-                 GObject   *gobj,
+handle_toggle_up(GObject   *gobj,
                  gboolean   gc_already_blocked)
 {
     ObjectInstance *priv;
@@ -884,7 +890,7 @@ handle_toggle_up(JSContext *context,
         goto out;
     }
 
-    priv = priv_from_js(context, obj);
+    priv = (ObjectInstance *) JS_GetPrivate(obj);
 
     gjs_debug_lifecycle(GJS_DEBUG_GOBJECT,
                         "Toggle notify gobj %p obj %p is_last_ref FALSEd keep-alive %p",
@@ -894,9 +900,12 @@ handle_toggle_up(JSContext *context,
      * in case the wrapper has data in it that the app cares about
      */
     if (priv->keep_alive == NULL) {
+        /* FIXME: thread the context through somehow. Maybe by looking up
+         * the compartment that obj belongs to. */
+        GjsContext *context = gjs_context_get_current();
         gjs_debug_lifecycle(GJS_DEBUG_GOBJECT, "Adding object to keep alive");
-        priv->keep_alive = gjs_keep_alive_get_global(context);
-        gjs_keep_alive_add_child(context, priv->keep_alive,
+        priv->keep_alive = gjs_keep_alive_get_global((JSContext*) gjs_context_get_native_context(context));
+        gjs_keep_alive_add_child(priv->keep_alive,
                                  gobj_no_longer_kept_alive_func,
                                  obj,
                                  priv);
@@ -919,10 +928,10 @@ idle_handle_toggle(gpointer data)
 
     switch (operation->direction) {
         case TOGGLE_UP:
-            handle_toggle_up(operation->context, operation->gobj, FALSE);
+            handle_toggle_up(operation->gobj, FALSE);
             break;
         case TOGGLE_DOWN:
-            handle_toggle_down(operation->context, operation->gobj);
+            handle_toggle_down(operation->gobj);
             break;
         default:
             g_assert_not_reached();
@@ -943,7 +952,6 @@ toggle_ref_notify_operation_free(ToggleRefNotifyOperation *operation)
 
 static void
 queue_toggle_idle(GObject         *gobj,
-                  JSContext       *context,
                   ToggleDirection  direction)
 {
     ToggleRefNotifyOperation *operation;
@@ -951,7 +959,6 @@ queue_toggle_idle(GObject         *gobj,
     GSource *source;
 
     operation = g_slice_new0(ToggleRefNotifyOperation);
-    operation->context = context;
     operation->direction = direction;
 
     switch (direction) {
@@ -1001,23 +1008,9 @@ wrapped_gobj_toggle_notify(gpointer      data,
                            GObject      *gobj,
                            gboolean      is_last_ref)
 {
-    JSRuntime *runtime;
-    JSContext *context;
     gboolean gc_blocked = FALSE;
     gboolean toggle_up_queued, toggle_down_queued;
 
-    runtime = (JSRuntime*) data;
-
-    /* During teardown, this can return NULL if runtime is being destroyed.
-     * In that case we effectively already converted to a weak ref without
-     * doing anything since the keep alive will be collected.
-     * Or if !is_last_ref, we do not want to convert to a strong
-     * ref since we want everything collected on runtime destroy.
-     */
-    context = gjs_runtime_get_context(runtime);
-    if (!context)
-        return;
-
     /* We only want to touch javascript from one thread.
      * If we're not in that thread, then we need to defer processing
      * to it. We also don't want to touch javascript if a GC is going
@@ -1046,9 +1039,9 @@ wrapped_gobj_toggle_notify(gpointer      data,
                         toggle_up_queued? "up" : "down");
             }
 
-            handle_toggle_down(context, gobj);
+            handle_toggle_down(gobj);
         } else {
-            queue_toggle_idle(gobj, context, TOGGLE_DOWN);
+            queue_toggle_idle(gobj, TOGGLE_DOWN);
         }
     } else {
         /* We've transitioned from 1 -> 2 references.
@@ -1062,9 +1055,9 @@ wrapped_gobj_toggle_notify(gpointer      data,
                         G_OBJECT_TYPE_NAME(gobj));
             }
 
-            handle_toggle_up(context, gobj, gc_blocked);
+            handle_toggle_up(gobj, gc_blocked);
         } else {
-            queue_toggle_idle(gobj, context, TOGGLE_UP);
+            queue_toggle_idle(gobj, TOGGLE_UP);
         }
     }
 
@@ -1144,15 +1137,12 @@ associate_js_gobject (JSContext      *context,
      * wrappee).
      */
     priv->keep_alive = gjs_keep_alive_get_global(context);
-    gjs_keep_alive_add_child(context,
-                             priv->keep_alive,
+    gjs_keep_alive_add_child(priv->keep_alive,
                              gobj_no_longer_kept_alive_func,
                              object,
                              priv);
 
-    g_object_add_toggle_ref(gobj,
-                            wrapped_gobj_toggle_notify,
-                            JS_GetRuntime(context));
+    g_object_add_toggle_ref(gobj, wrapped_gobj_toggle_notify, NULL);
 }
 
 static JSBool
@@ -1257,8 +1247,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(object_instance)
 
     GJS_NATIVE_CONSTRUCTOR_PRELUDE(object_instance);
 
-    object_init_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                                    GJS_STRING_GOBJECT_INIT);
+    object_init_name = gjs_context_get_const_string(context, GJS_STRING_GOBJECT_INIT);
     if (!gjs_object_require_property(context, object, "GObject instance", object_init_name, &initer))
         return JS_FALSE;
 
@@ -1342,8 +1331,7 @@ object_instance_finalize(JSFreeOp  *fop,
         }
 
         set_js_obj(priv->gobj, NULL);
-        g_object_remove_toggle_ref(priv->gobj, wrapped_gobj_toggle_notify,
-                                   fop->runtime());
+        g_object_remove_toggle_ref(priv->gobj, wrapped_gobj_toggle_notify, NULL);
         priv->gobj = NULL;
     }
 
@@ -1358,19 +1346,10 @@ object_instance_finalize(JSFreeOp  *fop,
         gjs_debug_lifecycle(GJS_DEBUG_GOBJECT,
                             "Removing from keep alive");
 
-        /* We're in a finalizer while the runtime is about to be
-         * destroyed. This is not the safest time to be calling back
-         * into jsapi, but we have to do this or the keep alive could
-         * be finalized later and call gobj_no_longer_kept_alive_func.
-         */
-        {
-            JSContext *context = JS_NewContext(fop->runtime(), 8192);
-            gjs_keep_alive_remove_child(context, priv->keep_alive,
-                                        gobj_no_longer_kept_alive_func,
-                                        obj,
-                                        priv);
-            JS_DestroyContext(context);
-        }
+        gjs_keep_alive_remove_child(priv->keep_alive,
+                                    gobj_no_longer_kept_alive_func,
+                                    obj,
+                                    priv);
     }
 
     if (priv->info) {
@@ -1773,7 +1752,7 @@ struct JSClass gjs_object_instance_class = {
     JSCLASS_HAS_PRIVATE |
     JSCLASS_NEW_RESOLVE,
     JS_PropertyStub,
-    JS_PropertyStub,
+    JS_DeletePropertyStub,
     object_instance_get_prop,
     object_instance_set_prop,
     JS_EnumerateStub,
@@ -1868,6 +1847,8 @@ gjs_define_object_class(JSContext      *context,
     JSObject *prototype;
     JSObject *constructor;
     JSObject *parent_proto;
+    JSObject *global;
+
     jsval value;
     ObjectInstance *priv;
     const char *ns;
@@ -1932,6 +1913,9 @@ gjs_define_object_class(JSContext      *context,
         constructor_name = g_type_name(gtype);
     }
 
+    global = JS_GetGlobalObject(context);
+    JSAutoCompartment ac(context, global);
+
     if (!gjs_init_class_dynamic(context, in_object,
                                 parent_proto,
                                 ns, constructor_name,
@@ -1991,6 +1975,7 @@ gjs_object_from_g_object(JSContext    *context,
                          GObject      *gobj)
 {
     JSObject *obj;
+    JSObject *global;
 
     if (gobj == NULL)
         return NULL;
@@ -2010,6 +1995,8 @@ gjs_object_from_g_object(JSContext    *context,
         proto = gjs_lookup_object_prototype(context, gtype);
 
         JS_BeginRequest(context);
+        global = JS_GetGlobalObject(context);
+        JSAutoCompartment ac(context, global);
 
         obj = JS_NewObjectWithGivenProto(context,
                                          JS_GetClass(proto), proto,
@@ -2228,7 +2215,7 @@ gjs_hook_up_vfunc(JSContext *cx,
 
     JS_SET_RVAL(cx, vp, JSVAL_VOID);
 
-    vfunc = find_vfunc_on_parent(info, name);
+    vfunc = find_vfunc_on_parents(info, name, NULL);
 
     if (!vfunc) {
         guint i, n_interfaces;
