@@ -29,7 +29,6 @@
 #include <gjs/gjs-module.h>
 #include <gjs/importer.h>
 #include <gjs/compat.h>
-#include <gjs/runtime.h>
 
 #include <gio/gio.h>
 
@@ -269,6 +268,8 @@ import_file(JSContext  *context,
     jsval script_retval;
     GError *error = NULL;
 
+    JS::CompileOptions options(context);
+
     if (!(g_file_load_contents(file, NULL, &script, &script_len, NULL, &error))) {
         if (!g_error_matches(error, G_IO_ERROR, G_IO_ERROR_IS_DIRECTORY) &&
             !g_error_matches(error, G_IO_ERROR, G_IO_ERROR_NOT_DIRECTORY) &&
@@ -284,30 +285,9 @@ import_file(JSContext  *context,
 
     full_path = g_file_get_parse_name (file);
 
-    if (!JS_EvaluateScript(context,
-                           module_obj,
-                           script,
-                           script_len,
-                           full_path,
-                           1, /* line number */
-                           &script_retval)) {
-
-        /* If JSOPTION_DONT_REPORT_UNCAUGHT is set then the exception
-         * would be left set after the evaluate and not go to the error
-         * reporter function.
-         */
-        if (JS_IsExceptionPending(context)) {
-            gjs_debug(GJS_DEBUG_IMPORTER,
-                      "Module '%s' left an exception set",
-                      name);
-            gjs_log_and_keep_exception(context);
-        } else {
-            gjs_throw(context,
-                         "JS_EvaluateScript() returned FALSE but did not set exception");
-        }
-
+    if (!gjs_eval_with_scope(context, module_obj, script, script_len,
+                             full_path, NULL))
         goto out;
-    }
 
     ret = JS_TRUE;
 
@@ -328,8 +308,7 @@ load_module_init(JSContext  *context,
     GFile *file;
 
     /* First we check if js module has already been loaded  */
-    module_init_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                                    GJS_STRING_MODULE_INIT);
+    module_init_name = gjs_context_get_const_string(context, GJS_STRING_MODULE_INIT);
     if (JS_HasPropertyById(context, in_object, module_init_name, &found) && found) {
         jsval module_obj_val;
 
@@ -452,8 +431,7 @@ do_import(JSContext  *context,
     GFile *gfile;
     gboolean exists;
 
-    search_path_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                                    GJS_STRING_SEARCH_PATH);
+    search_path_name = gjs_context_get_const_string(context, GJS_STRING_SEARCH_PATH);
     if (!gjs_object_require_property(context, obj, "importer", search_path_name, &search_path_val)) {
         return JS_FALSE;
     }
@@ -713,8 +691,7 @@ importer_new_enumerate(JSContext  *context,
             /* we are enumerating the prototype properties */
             return JS_TRUE;
 
-        search_path_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                                        GJS_STRING_SEARCH_PATH);
+        search_path_name = gjs_context_get_const_string(context, GJS_STRING_SEARCH_PATH);
         if (!gjs_object_require_property(context, *object, "importer", search_path_name, &search_path_val))
             return JS_FALSE;
 
@@ -890,8 +867,7 @@ importer_new_resolve(JSContext *context,
 
     *objp = NULL;
 
-    module_init_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                                    GJS_STRING_MODULE_INIT);
+    module_init_name = gjs_context_get_const_string(context, GJS_STRING_MODULE_INIT);
     if (*id == module_init_name)
         return JS_TRUE;
 
@@ -949,7 +925,7 @@ struct JSClass gjs_importer_class = {
     JSCLASS_NEW_RESOLVE |
     JSCLASS_NEW_ENUMERATE,
     JS_PropertyStub,
-    JS_PropertyStub,
+    JS_DeletePropertyStub,
     JS_PropertyStub,
     JS_StrictPropertyStub,
     (JSEnumerateOp) importer_new_enumerate, /* needs cast since it's the new enumerate signature */
@@ -1058,16 +1034,8 @@ gjs_get_search_path(void)
             g_free(dirs);
         }
 
-        /* $XDG_DATA_DIRS /gjs-1.0 */
-        system_data_dirs = g_get_system_data_dirs();
-        for (i = 0; system_data_dirs[i] != NULL; ++i) {
-            char *s;
-            s = g_build_filename(system_data_dirs[i], "gjs-1.0", NULL);
-            g_ptr_array_add(path, s);
-        }
-
         /* ${datadir}/share/gjs-1.0 */
-        g_ptr_array_add(path, g_strdup(GJS_JS_DIR));
+        g_ptr_array_add(path, g_strdup("resource:///org/gnome/gjs/modules/"));
 
         g_ptr_array_add(path, NULL);
 
@@ -1187,8 +1155,7 @@ gjs_define_root_importer(JSContext   *context,
     JS_BeginRequest(context);
 
     importer = gjs_get_global_slot(context, GJS_GLOBAL_SLOT_IMPORTS);
-    imports_name = gjs_runtime_get_const_string(JS_GetRuntime(context),
-                                                GJS_STRING_IMPORTS);
+    imports_name = gjs_context_get_const_string(context, GJS_STRING_IMPORTS);
     if (!JS_DefinePropertyById(context, in_object,
                                imports_name, importer,
                                NULL, NULL,
