@@ -27,11 +27,11 @@
 #include <util/log.h>
 #include <util/glib.h>
 #include <util/misc.h>
+#include <util/error.h>
 
 #include "jsapi-util.h"
 #include "compat.h"
 #include "jsapi-private.h"
-#include "runtime.h"
 #include <gi/boxed.h>
 
 #include <string.h>
@@ -72,7 +72,7 @@ gjs_get_import_global(JSContext *context)
 
 static JSClass global_class = {
     "GjsGlobal", JSCLASS_GLOBAL_FLAGS_WITH_SLOTS(GJS_GLOBAL_SLOT_LAST),
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
+    JS_PropertyStub, JS_DeletePropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, NULL,
     NULL /* checkAccess */, NULL /* call */, NULL /* hasInstance */, NULL /* construct */, NULL,
     { NULL }
@@ -88,16 +88,46 @@ static JSClass global_class = {
  * Returns: %TRUE on success, %FALSE otherwise
  */
 gboolean
-gjs_init_context_standard (JSContext       *context)
+gjs_init_context_standard (JSContext *context)
 {
     JSObject *global;
+    JS::CompartmentOptions options;
+    guint32 options_flags;
 
-    global = JS_NewGlobalObject(context, &global_class, NULL);
+    /* JSOPTION_DONT_REPORT_UNCAUGHT: Don't send exceptions to our
+     * error report handler; instead leave them set.  This allows us
+     * to get at the exception object.
+     *
+     * JSOPTION_STRICT: Report warnings to error reporter function.
+     */
+    options_flags = JSOPTION_DONT_REPORT_UNCAUGHT | JSOPTION_EXTRA_WARNINGS;
+
+    if (!g_getenv("GJS_DISABLE_JIT")) {
+        gjs_debug(GJS_DEBUG_CONTEXT, "Enabling JIT");
+        options_flags |= JSOPTION_TYPE_INFERENCE | JSOPTION_ION | JSOPTION_BASELINE | JSOPTION_ASMJS;
+    }
+
+    JS_SetOptions(context, JS_GetOptions(context) | options_flags);
+    JS_SetErrorReporter(context, gjs_error_reporter);
+
+    options.setVersion(JSVERSION_LATEST);
+    global = JS_NewGlobalObject(context, &global_class, NULL, options);
     if (global == NULL)
         return FALSE;
+
+    /* Set the context's global */
+    JSAutoCompartment ac(context, global);
+    JS_SetGlobalObject(context, global);
+
     if (!JS_InitStandardClasses(context, global))
         return FALSE;
 
+    if (!JS_InitReflect(context, global))
+        return FALSE;
+
+    if (!JS_DefineDebuggerObject(context, global))
+        return FALSE;
+
     return TRUE;
 }
 
@@ -338,7 +368,7 @@ gjs_value_debug_string(JSContext      *context,
     size_t len = JS_GetStringEncodingLength(context, str);
     if (len != (size_t)(-1)) {
         bytes = (char*) g_malloc((len + 1) * sizeof(char));
-        JS_EncodeStringToBuffer(str, bytes, len);
+        JS_EncodeStringToBuffer(context, str, bytes, len);
         bytes[len] = '\0';
     } else {
         bytes = g_strdup("[invalid string]");
@@ -457,10 +487,13 @@ gjs_log_exception_full(JSContext *context,
 {
     jsval stack;
     JSString *exc_str;
-    char *utf8_exception, *utf8_message, *utf8_stack;
+    char *utf8_exception, *utf8_message;
+    gboolean is_syntax;
 
     JS_BeginRequest(context);
 
+    is_syntax = FALSE;
+
     if (JSVAL_IS_OBJECT(exc) &&
         gjs_typecheck_boxed(context, JSVAL_TO_OBJECT(exc), NULL, G_TYPE_ERROR, FALSE)) {
         GError *gerror;
@@ -470,6 +503,18 @@ gjs_log_exception_full(JSContext *context,
                                          g_quark_to_string(gerror->domain),
                                          gerror->message);
     } else {
+        if (JSVAL_IS_OBJECT(exc)) {
+            jsval js_name;
+            char *utf8_name;
+
+            if (gjs_object_get_property_const(context, JSVAL_TO_OBJECT(exc),
+                                              GJS_STRING_NAME, &js_name) &&
+                JSVAL_IS_STRING(js_name) &&
+                gjs_string_to_utf8(context, js_name, &utf8_name)) {
+                is_syntax = strcmp("SyntaxError", utf8_name) == 0;
+            }
+        }
+
         exc_str = JS_ValueToString(context, exc);
         if (exc_str != NULL)
             gjs_string_to_utf8(context, STRING_TO_JSVAL(exc_str), &utf8_exception);
@@ -482,29 +527,65 @@ gjs_log_exception_full(JSContext *context,
     else
         utf8_message = NULL;
 
-    if (JSVAL_IS_OBJECT(exc) &&
-        gjs_object_get_property_const(context, JSVAL_TO_OBJECT(exc),
-                                      GJS_STRING_STACK, &stack) &&
-        JSVAL_IS_STRING(stack))
-        gjs_string_to_utf8(context, stack, &utf8_stack);
-    else
-        utf8_stack = NULL;
+    /* We log syntax errors differently, because the stack for those includes
+       only the referencing module, but we want to print out the filename and
+       line number from the exception.
+    */
 
-    if (utf8_message) {
-        if (utf8_stack)
-            g_warning("JS ERROR: %s: %s\n%s", utf8_message, utf8_exception, utf8_stack);
+    if (is_syntax) {
+        jsval js_lineNumber, js_fileName;
+        unsigned lineNumber;
+        char *utf8_fileName;
+
+        gjs_object_get_property_const(context, JSVAL_TO_OBJECT(exc),
+                                      GJS_STRING_LINE_NUMBER, &js_lineNumber);
+        gjs_object_get_property_const(context, JSVAL_TO_OBJECT(exc),
+                                      GJS_STRING_FILENAME, &js_fileName);
+
+        if (JSVAL_IS_STRING(js_fileName))
+            gjs_string_to_utf8(context, js_fileName, &utf8_fileName);
         else
-            g_warning("JS ERROR: %s: %s", utf8_message, utf8_exception);
+            utf8_fileName = g_strdup("unknown");
+
+        lineNumber = JSVAL_TO_INT(js_lineNumber);
+
+        if (utf8_message) {
+            g_critical("JS ERROR: %s: %s @ %s:%u", utf8_message, utf8_exception,
+                       utf8_fileName, lineNumber);
+        } else {
+            g_critical("JS ERROR: %s @ %s:%u", utf8_exception,
+                       utf8_fileName, lineNumber);
+        }
+
+        g_free(utf8_fileName);
     } else {
-        if (utf8_stack)
-            g_warning("JS ERROR: %s\n%s", utf8_exception, utf8_stack);
+        char *utf8_stack;
+
+        if (JSVAL_IS_OBJECT(exc) &&
+            gjs_object_get_property_const(context, JSVAL_TO_OBJECT(exc),
+                                          GJS_STRING_STACK, &stack) &&
+            JSVAL_IS_STRING(stack))
+            gjs_string_to_utf8(context, stack, &utf8_stack);
         else
-            g_warning("JS ERROR: %s", utf8_exception);
+            utf8_stack = NULL;
+
+        if (utf8_message) {
+            if (utf8_stack)
+                g_warning("JS ERROR: %s: %s\n%s", utf8_message, utf8_exception, utf8_stack);
+            else
+                g_warning("JS ERROR: %s: %s", utf8_message, utf8_exception);
+        } else {
+            if (utf8_stack)
+                g_warning("JS ERROR: %s\n%s", utf8_exception, utf8_stack);
+            else
+                g_warning("JS ERROR: %s", utf8_exception);
+        }
+
+        g_free(utf8_stack);
     }
 
     g_free(utf8_exception);
     g_free(utf8_message);
-    g_free(utf8_stack);
 
     JS_EndRequest(context);
 
@@ -738,35 +819,6 @@ gjs_get_type_name(jsval value)
     }
 }
 
-jsval
-gjs_date_from_time_t (JSContext *context, time_t time)
-{
-    JSObject *date;
-    JSObject *date_constructor;
-    jsval date_prototype;
-    jsval args[1];
-    jsval result;
-
-    JS_BeginRequest(context);
-
-    if (!JS_GetClassObject(context, JS_GetGlobalObject(context), JSProto_Date,
-                           &date_constructor))
-        g_error("Failed to lookup Date prototype");
-
-    if (!JS_GetProperty(context, date_constructor, "prototype", &date_prototype))
-        g_error("Failed to get prototype from Date constructor");
-
-    if (!JS_NewNumberValue(context, ((double) time) * 1000, &(args[0])))
-        g_error("Failed to convert time_t to number");
-
-    date = JS_New(context, JSVAL_TO_OBJECT (date_prototype), 1, args);
-
-    result = OBJECT_TO_JSVAL(date);
-    JS_EndRequest(context);
-
-    return result;
-}
-
 /**
  * gjs_value_to_int64:
  * @context: the Javascript context object
@@ -1158,3 +1210,110 @@ gjs_unblock_gc(void)
 {
     g_mutex_unlock(&gc_lock);
 }
+
+/**
+ * gjs_strip_unix_shebang:
+ *
+ * @script: (in): A pointer to a JS script
+ * @script_len: (inout): A pointer to the script length. The
+ * pointer will be modified if a shebang is stripped.
+ * @new_start_line_number: (out) (allow-none): A pointer to
+ * write the start-line number to account for the offset
+ * as a result of stripping the shebang.
+ *
+ * Returns a pointer to the beginning of a script with unix
+ * shebangs removed. The outparams are useful to know the
+ * new length of the script and on what line of the
+ * original script we're executing from, so that any relevant
+ * offsets can be applied to the results of an execution pass.
+ */
+const char *
+gjs_strip_unix_shebang(const char  *script,
+                       gssize      *script_len,
+                       int         *start_line_number_out)
+{
+    g_assert(script_len);
+
+    /* handle scripts with UNIX shebangs */
+    if (strncmp(script, "#!", 2) == 0) {
+        /* If we found a newline, advance the script by one line */
+        const char *s = (const char *) strstr (script, "\n");
+        if (s != NULL) {
+            if (*script_len > 0)
+                *script_len -= (s + 1 - script);
+            script = s + 1;
+
+            if (start_line_number_out)
+                *start_line_number_out = 2;
+
+            return script;
+        } else {
+            /* Just a shebang */
+            if (start_line_number_out)
+                *start_line_number_out = -1;
+
+            *script_len = 0;
+
+            return NULL;
+        }
+    }
+
+    /* No shebang, return the original script */
+    if (start_line_number_out)
+        *start_line_number_out = 1;
+
+    return script;
+}
+
+JSBool
+gjs_eval_with_scope(JSContext    *context,
+                    JSObject     *object,
+                    const char   *script,
+                    gssize        script_len,
+                    const char   *filename,
+                    jsval        *retval_p)
+{
+    int start_line_number = 1;
+    jsval retval = JSVAL_VOID;
+    JSAutoRequest ar(context);
+
+    if (script_len < 0)
+        script_len = strlen(script);
+
+    script = gjs_strip_unix_shebang(script,
+                                    &script_len,
+                                    &start_line_number);
+
+    /* log and clear exception if it's set (should not be, normally...) */
+    if (JS_IsExceptionPending(context)) {
+        g_warning("gjs_eval_in_scope called with a pending exception");
+        return JS_FALSE;
+    }
+
+    if (!object)
+        object = JS_NewObject(context, NULL, NULL, NULL);
+
+    JS::CompileOptions options(context);
+    options.setUTF8(true)
+           .setFileAndLine(filename, start_line_number)
+           .setSourcePolicy(JS::CompileOptions::LAZY_SOURCE);
+
+    js::RootedObject rootedObj(context, object);
+
+    if (!JS::Evaluate(context, rootedObj, options, script, script_len, &retval))
+        return JS_FALSE;
+
+    if (JS_IsExceptionPending(context)) {
+        g_warning("EvaluateScript returned JS_TRUE but exception was pending; "
+                  "did somebody call gjs_throw() without returning JS_FALSE?");
+        return JS_FALSE;
+    }
+
+    gjs_debug(GJS_DEBUG_CONTEXT,
+              "Script evaluation succeeded");
+
+    if (retval_p)
+        *retval_p = retval;
+
+    return JS_TRUE;
+}
