@@ -31,6 +31,7 @@
 #include <gjs/compat.h>
 #include <gjs/runtime.h>
 #include <glib-object.h>
+#include <gi/gtype.h>
 
 G_BEGIN_DECLS
 
@@ -47,6 +48,7 @@ enum {
 typedef enum {
     GJS_GLOBAL_SLOT_IMPORTS,
     GJS_GLOBAL_SLOT_KEEP_ALIVE,
+    GJS_GLOBAL_SLOT_BYTE_ARRAY_PROTOTYPE,
     GJS_GLOBAL_SLOT_LAST,
 } GjsGlobalSlot;
 
@@ -112,7 +114,7 @@ typedef struct GjsRootedArray GjsRootedArray;
  */
 #define GJS_DEFINE_PROTO(tn, cn) \
 GJS_NATIVE_CONSTRUCTOR_DECLARE(cn); \
-_GJS_DEFINE_PROTO_FULL(tn, cn, gjs_##cn##_constructor)
+_GJS_DEFINE_PROTO_FULL(tn, cn, gjs_##cn##_constructor, G_TYPE_NONE)
 
 /**
  * GJS_DEFINE_PROTO_ABSTRACT:
@@ -124,9 +126,16 @@ _GJS_DEFINE_PROTO_FULL(tn, cn, gjs_##cn##_constructor)
  * you won't be able to instantiate it using the new keyword
  */
 #define GJS_DEFINE_PROTO_ABSTRACT(tn, cn) \
-_GJS_DEFINE_PROTO_FULL(tn, cn, NULL)
+_GJS_DEFINE_PROTO_FULL(tn, cn, NULL, G_TYPE_NONE)
 
-#define _GJS_DEFINE_PROTO_FULL(type_name, cname, ctor) \
+#define GJS_DEFINE_PROTO_WITH_GTYPE(tn, cn, gtype)   \
+GJS_NATIVE_CONSTRUCTOR_DECLARE(cn); \
+_GJS_DEFINE_PROTO_FULL(tn, cn, gjs_##cn##_constructor, gtype)
+
+#define GJS_DEFINE_PROTO_ABSTRACT_WITH_GTYPE(tn, cn, gtype)   \
+_GJS_DEFINE_PROTO_FULL(tn, cn, NULL, gtype)
+
+#define _GJS_DEFINE_PROTO_FULL(type_name, cname, ctor, gtype) \
 extern JSPropertySpec gjs_##cname##_proto_props[]; \
 extern JSFunctionSpec gjs_##cname##_proto_funcs[]; \
 static void gjs_##cname##_finalize(JSFreeOp *fop, JSObject *obj); \
@@ -143,7 +152,7 @@ static struct JSClass gjs_##cname##_class = { \
     JSCLASS_HAS_PRIVATE | \
     JSCLASS_NEW_RESOLVE, \
     JS_PropertyStub, \
-    JS_PropertyStub, \
+    JS_DeletePropertyStub, \
     JS_PropertyStub, \
     JS_StrictPropertyStub, \
     JS_EnumerateStub,\
@@ -162,7 +171,8 @@ jsval gjs_##cname##_create_proto(JSContext *context, JSObject *module, const cha
     if (!JS_GetPropertyById(context, global, class_name, &rval))                       \
         return JSVAL_NULL; \
     if (JSVAL_IS_VOID(rval)) { \
-        JSObject *prototype = JS_InitClass(context, global, \
+        jsval value; \
+        JSObject *prototype = JS_InitClass(context, global,     \
                                  parent, \
                                  &gjs_##cname##_class, \
                                  ctor, \
@@ -182,6 +192,11 @@ jsval gjs_##cname##_create_proto(JSContext *context, JSObject *module, const cha
         if (!JS_DefineProperty(context, module, proto_name, \
                                rval, NULL, NULL, GJS_MODULE_PROP_FLAGS)) \
             return JSVAL_NULL; \
+        if (gtype != G_TYPE_NONE) { \
+            value = OBJECT_TO_JSVAL(gjs_gtype_create_gtype_wrapper(context, gtype)); \
+            JS_DefineProperty(context, JSVAL_TO_OBJECT(rval), "$gtype", value, \
+                              NULL, NULL, JSPROP_PERMANENT);            \
+        } \
     } \
     return rval; \
 }
@@ -201,12 +216,6 @@ gboolean    gjs_object_require_property      (JSContext       *context,
                                               const char      *obj_description,
                                               jsid             property_name,
                                               jsval           *value_p);
-/* This one is defined in runtime.c, so the compiler can optimize the call
-   to get_const_string() it uses. */
-gboolean    gjs_object_get_property_const    (JSContext       *context,
-                                              JSObject        *obj,
-                                              GjsConstString   property_name,
-                                              jsval           *value_p);
 
 JSObject   *gjs_new_object_for_constructor   (JSContext       *context,
                                               JSClass         *clasp,
@@ -283,6 +292,7 @@ JSBool      gjs_call_function_value          (JSContext       *context,
 void        gjs_error_reporter               (JSContext       *context,
                                               const char      *message,
                                               JSErrorReport   *report);
+JSObject*   gjs_get_global_object            (JSContext *cx);
 JSBool      gjs_get_prop_verbose_stub        (JSContext       *context,
                                               JSObject        *obj,
                                               jsval            id,
@@ -334,8 +344,6 @@ JSBool      gjs_value_to_int64               (JSContext       *context,
                                               const jsval      val,
                                               gint64          *result);
 
-jsval       gjs_date_from_time_t             (JSContext *context, time_t time);
-
 JSBool      gjs_parse_args                   (JSContext  *context,
                                               const char *function_name,
                                               const char *format,
@@ -382,6 +390,49 @@ JSBool            gjs_context_get_frame_info (JSContext  *context,
                                               jsval      *fileName,
                                               jsval      *lineNumber);
 
+JSBool            gjs_eval_with_scope        (JSContext    *context,
+                                              JSObject     *object,
+                                              const char   *script,
+                                              gssize        script_len,
+                                              const char   *filename,
+                                              jsval        *retval_p);
+
+typedef enum {
+  GJS_STRING_CONSTRUCTOR,
+  GJS_STRING_PROTOTYPE,
+  GJS_STRING_LENGTH,
+  GJS_STRING_IMPORTS,
+  GJS_STRING_PARENT_MODULE,
+  GJS_STRING_MODULE_INIT,
+  GJS_STRING_SEARCH_PATH,
+  GJS_STRING_KEEP_ALIVE_MARKER,
+  GJS_STRING_PRIVATE_NS_MARKER,
+  GJS_STRING_GI_MODULE,
+  GJS_STRING_GI_VERSIONS,
+  GJS_STRING_GI_OVERRIDES,
+  GJS_STRING_GOBJECT_INIT,
+  GJS_STRING_NEW_INTERNAL,
+  GJS_STRING_NEW,
+  GJS_STRING_MESSAGE,
+  GJS_STRING_CODE,
+  GJS_STRING_STACK,
+  GJS_STRING_FILENAME,
+  GJS_STRING_LINE_NUMBER,
+  GJS_STRING_NAME,
+  GJS_STRING_LAST
+} GjsConstString;
+
+jsid              gjs_context_get_const_string  (JSContext       *context,
+                                                 GjsConstString   string);
+gboolean          gjs_object_get_property_const (JSContext       *context,
+                                                 JSObject        *obj,
+                                                 GjsConstString   property_name,
+                                                 jsval           *value_p);
+
+const char * gjs_strip_unix_shebang(const char *script,
+                                    gssize     *script_len,
+                                    int        *new_start_line_number);
+
 G_END_DECLS
 
 #endif  /* __GJS_JSAPI_UTIL_H__ */
